import java.io.IOException;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.SequenceFile.Reader;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;
public class KMeansMapper extends
                        Mapper<LongWritable, Text, IntWritable, LongWritable> {
                String BASE_PATH = "/user/zhidong/TCGAkmeans";

                private static Map<Integer, Long> centers = new HashMap<Integer, Long>();
                private static double betaValueThreshod=0.5;
                private IntWritable classCenter;
                private int clusterNum=10;

                @Override
                protected void setup(Context context) throws IOException,
                                InterruptedException 
                  {
                        Configuration conf = context.getConfiguration();
                        Path centroids = new Path(conf.get("CENPATH"));
                
                        FileSystem fs = FileSystem.get(conf);

                        if(!fs.exists(centroids))
                        {
                              //Initialize the initial cluster centers If file not exists
                             System.out.println("No center informations. Generate automatically"); 
                             for(int i=0;i<30;i++)
                             {
                                  for(int j=0;j<clusterNum;j++)
                                  {
                                       //It uses a straight forward way, but later it will be changed. 
                                       centers.put(new Integer(i*10+j),new Long(j*20000));
                                  }
                             }

                        }
                       else
                       {

                        SequenceFile.Reader reader = new SequenceFile.Reader(fs, centroids,
                                        conf);
                        
                        IntWritable key = new IntWritable();
                        LongWritable value = new LongWritable();
                        while (reader.next(key, value)) {
                               
                                if(key!=null&&value!=null)
                                centers.put(new Integer(key.get()),new Long(value.get()));
                        }
                        reader.close();
            
                      }
                        super.setup(context);
                }
                @Override
                protected void map(LongWritable key, Text value, Context context)
                                throws IOException, InterruptedException {
                        int nearestNum = 0;
                        String line= value.toString();
                        String[] tokens = line.split("\\s");
                        for(int i=0;i<tokens.length;i++)tokens[i]=tokens[i].trim();
                        if(!tokens[0].subSequence(0, 2).equals("cg")) return;
                        if(tokens[1].equals("NA")||Double.parseDouble((tokens[1]))<betaValueThreshod)return; // We only cluster gene with betaValue larger than threshold. 
                        if(tokens[3].equals("NA"))return;
                        if(tokens[4].equals("NA"))return;
                        long genePosition= Long.parseLong(tokens[4]);
                        int chromosomeNum;
                        int maxClusters= clusterNum;
                        if(tokens[3].equals("X")||tokens[3].equals("Y"))
                        {
                            chromosomeNum=23;
                        }
                        else { chromosomeNum=Integer.parseInt(tokens[3]); }
                        int base=chromosomeNum*maxClusters;
                        int  minIndex=0;
                        long  minDistance=1000000000;
                        for(int i=0;i<clusterNum;i++)
                        {
                            if(centers.containsKey(base+i))
                          {
                            if(Math.abs(centers.get(base+i)-genePosition)<minDistance)
                            {
                               
                                    minDistance=Math.abs(centers.get(base+i)-genePosition);
                                    minIndex=i;
                               
                            }
                          }
                        }

                        int classNumEncoded= base+minIndex;

                        this.classCenter = new IntWritable(classNumEncoded);
                        context.write(this.classCenter, new LongWritable(genePosition));
                }

        }

